/*==============================================================================


   Purpose:    determine MHz rate of processor


   Author:     Rudolf Berrendorf
               University of Applied Sciences Rhein-Sieg
               Computer Science Department
               53754 Sankt Augustin
               rudolf.berrendorf@fh-rhein-sieg.de


   Date:       18.05.1999


   DISCLAIMER

                  PCL - The Performance Counter Library


   Copyright (C) 1999, Forschungszentrum Juelich GmbH, Federal Republic of
   Germany. All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

     - Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

     - Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

     - Any publications that result from the use of this software shall
       reasonably refer to the Research Centre's development.

     - All advertising materials mentioning features or use of this software
       must display the following acknowledgement:

           This product includes software developed by Forschungszentrum
           Juelich GmbH, Federal Republic of Germany.

     - Forschungszentrum Juelich GmbH is not obligated to provide the user with
       any support, consulting, training or assistance of any kind with regard
       to the use, operation and performance of this software or to provide
       the user with any updates, revisions or new versions.


   THIS SOFTWARE IS PROVIDED BY FORSCHUNGSZENTRUM JUELICH GMBH "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL FORSCHUNGSZENTRUM JUELICH GMBH BE LIABLE FOR
   ANY SPECIAL, DIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
   CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
   CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.

==============================================================================*/


#include <pcl.h>

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>


/* time to measure (in sec) */
#define NSECOND 0.01


/*                                                                            */
/*============================================================================*/
/* calculate the difference of two timeval numbers.                           */


double timediff(struct timeval tv2, struct timeval tv1)
{
  return (double)(tv2.tv_sec - tv1.tv_sec)
    + ((double)(tv2.tv_usec - tv1.tv_usec) / 1000000.0);
}


/*                                                                            */
/*============================================================================*/
/* main program.                                                              */


int
main(int argc, char **argv)
{
  PCL_CNT_TYPE i_result;
  PCL_FP_CNT_TYPE fp_result;
  int counter_list, res;
  unsigned int flags;
  struct timeval tv1, tv2;
  double t_delta;
  PCL_DESCR_TYPE descr;


  /* define events to be measured */
  counter_list = PCL_ELAPSED_CYCLES;

  /* define in what mode to count
     (user and system mode, but that dosn't care here) */
  flags = PCL_MODE_USER_SYSTEM;

  /* initialize */
  if(PCLinit(&descr) != PCL_SUCCESS)
    {
      printf("PCLinit not possible\n");
      exit(1);
    }

  /* start counting */
  if ((res = PCLstart(descr, &counter_list, 1, flags)) != PCL_SUCCESS)
    {
      printf("elapsed cycles not possible\n");
      exit(1);
    }

  /* do work */
  gettimeofday(&tv1, NULL);
  do
    {
      gettimeofday(&tv2, NULL);
    }
  while((t_delta = timediff(tv2, tv1)) < NSECOND);
  
  /* stop counting */
  if ((res = PCLstop(descr, &i_result, &fp_result, 1)) != PCL_SUCCESS)
    printf("problem with stopping\n");
  else
    printf("%.0f elapsed cycles in %f seconds: %.3f MHz\n",
	   (double) i_result, NSECOND, (double)i_result/t_delta/1000000.0);

  if(PCLexit(descr) != PCL_SUCCESS)
    {
      printf("PCLexit not successful\n");
      exit(1);
    }

  return 0;
}

/*============================================================================*
 *                             that's all folks                               *
 *============================================================================*/
